#include "functional/common.h"
#include "tensor_utils.h"

#include <algorithm>
#include <functional>
#include <numeric>
#include <stdexcept>

// TODO: this approach does not scale well.
// The right way is probably implementing some type_traits for the Tensor_type_t
// enum
template <> float *convertAndGetGPUData<float>(Tensor *t) {
  if (t->cur_type == float_type)
    return static_cast<float *>(t->gpu_data);
  if (t->cur_type == half_type) {
    convertToFP32(t);
    t->data_type = float_type;
    return static_cast<float *>(t->gpu_data);
  }
  ERROR("Type %s is incompatible with target type float\n",
        std::to_string(t->cur_type));
}

template <> half *convertAndGetGPUData<half>(Tensor *t) {
  if (t->cur_type == half_type)
    return static_cast<half *>(t->gpu_half_data);
  if (t->cur_type == float_type) {
    convertToFP16(t);
    t->data_type = half_type;
    return static_cast<half *>(t->gpu_half_data);
  }
  ERROR("Type %s is incompatible with target type half\n",
        std::to_string(t->cur_type));
}

template <> float2 *convertAndGetGPUData<float2>(Tensor *t) {
  if (t->cur_type == float2_type)
    return static_cast<float2 *>(t->gpu_data);
  if (t->cur_type == half2_type) {
    // FIXME: hacking to make convertToFP16 realize these are "2 floats"
    t->num_elems *= 2;
    convertToFP32(t);
    t->num_elems /= 2;
    t->cur_type = t->data_type = float2_type;
    return static_cast<float2 *>(t->gpu_data);
  }
  ERROR("Type %s is incompatible with target type float2\n",
        std::to_string(t->cur_type));
}

template <> half2 *convertAndGetGPUData<half2>(Tensor *t) {
  if (t->cur_type == half2_type)
    return static_cast<half2 *>(t->gpu_half_data);
  if (t->cur_type == float2_type) {
    // FIXME: hacking to make convertToFP16 realize these are "2 floats"
    t->num_elems *= 2;
    convertToFP16(t);
    t->num_elems /= 2;
    t->cur_type = t->data_type = half2_type;
    return static_cast<half2 *>(t->gpu_half_data);
  }
  ERROR("Type %s is incompatible with target type half2\n",
        std::to_string(t->cur_type));
}

void convertToFloat2Offline(Tensor *t) {
  if (t->cur_type == float2_type)
    return;
  else if (t->cur_type == half2_type) {
    t->cur_type = t->data_type = half_type;
    t->num_elems *= 2;
    convertToFP32_offline(t);
    t->num_elems /= 2;
    t->cur_type = t->data_type = float2_type;
  } else {
    ERROR("Type %s is incompatible with target type half2\n",
          std::to_string(t->cur_type));
  }
}

std::vector<size_t> sizes(const Dimension &dim) {
  return std::vector<size_t>(dim.dim_sizes, dim.dim_sizes + dim.num_dims);
}

std::vector<size_t> sizes(Tensor *t) { return sizes(t->dims); }

size_t num_elems(const std::vector<size_t> &dim_sizes) {
  return std::accumulate(dim_sizes.begin(), dim_sizes.end(), 1,
                         std::multiplies<>());
}

size_t num_elems(const Dimension &dim) { return num_elems(sizes(dim)); }

size_t num_elems(Tensor *t) { return num_elems(sizes(t)); }

static Tensor_type_t toHalfType(Tensor_type_t float_ty) {
  switch (float_ty) {
  case float_type:
    return half_type;
  case float2_type:
    return half2_type;
  case half_type:
  case half2_type:
    return float_ty;
  default:
    ERROR("Types not acceptable\n");
  }
}

static Tensor_type_t toFloatType(Tensor_type_t half_ty) {
  switch (half_ty) {
  case half_type:
    return float_type;
  case half2_type:
    return float2_type;
  case float_type:
  case float2_type:
    return half_ty;
  default:
    ERROR("Types not acceptable\n");
  }
}

Tensor_type_t getCompatibleType(int t1, int t2, bool get_half) {
  auto type1 = (Tensor_type_t)t1, type2 = (Tensor_type_t)t2;
  if (getTypeSize(type1) > getTypeSize(type2))
    std::swap(type1, type2);
  if (type1 == type2)
    return get_half ? toHalfType(type1)
                    : toFloatType(type1); // Or type2, whatever
  if (type1 == half_type && type2 == float_type)
    return get_half ? half_type : float_type;
  if (type1 == half2_type && type2 == float2_type)
    return get_half ? half2_type : float2_type;
  ERROR("Types not acceptable\n");
}
